package com.suntray.storage.config;

import com.fasterxml.jackson.databind.ObjectMapper;
import com.fasterxml.jackson.datatype.jsr310.deser.LocalDateTimeDeserializer;
import com.fasterxml.jackson.datatype.jsr310.ser.LocalDateSerializer;
import com.suntray.annotation.Log;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.lang3.StringUtils;
import org.aspectj.lang.ProceedingJoinPoint;
import org.aspectj.lang.annotation.Around;
import org.aspectj.lang.annotation.Aspect;
import org.aspectj.lang.annotation.Pointcut;
import org.aspectj.lang.reflect.CodeSignature;
import org.aspectj.lang.reflect.MethodSignature;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.http.converter.json.Jackson2ObjectMapperBuilder;
import org.springframework.stereotype.Component;

import java.lang.reflect.Method;
import java.text.MessageFormat;
import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;
import java.util.HashMap;
import java.util.Map;
import java.util.TimeZone;


/**
 * 日志拦截器，打印请求参数
 */
@Aspect
@Component
public class LogAspect {

    private Logger logger = LoggerFactory.getLogger(LogAspect.class);

    private static ObjectMapper objectMapper;

    static {
        DateTimeFormatter formatter = DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss");
        objectMapper = new Jackson2ObjectMapperBuilder().timeZone(TimeZone.getTimeZone("GMT+8"))
                .findModulesViaServiceLoader(true)
                .serializerByType(LocalDateTime.class, new LocalDateSerializer(formatter))
                .deserializerByType(LocalDateTime.class, new LocalDateTimeDeserializer(formatter))
                .build();
    }

    @Pointcut("@annotation(com.suntray.annotation.Log)")
    public void log() {

    }

    @Around("log()")
    public Object around(ProceedingJoinPoint point) throws Throwable {
        try {
            MethodSignature signature = (MethodSignature) point.getSignature();
            Method method = signature.getMethod();

            String className = point.getTarget().getClass().getName();
            String[] paramNames = ((CodeSignature) point.getSignature()).getParameterNames();
            Map<String, Object> params = new HashMap<>();
            if (paramNames != null && paramNames.length > 0) {
                Object[] args = point.getArgs();
                for (int i = 0; i < paramNames.length; i++) {
                    Object arg = args[i];
                    if (arg==null||arg.getClass().getPackage().getName().startsWith("org.springframework")) {
                        continue;
                    }
                    params.put(paramNames[i], arg);
                }
            }
            Log ann = method.getAnnotation(Log.class);
            String action = ann.value();
            if(StringUtils.isEmpty(ann.value())||method.isAnnotationPresent(ApiOperation.class)){
                action=method.getAnnotation(ApiOperation.class).value();
            }
            String content = MessageFormat.format("{0}，请求方法:{1}.{2}，参数：{3}", action, className, method.getName(), objectMapper.writeValueAsString(params));
            logger.info(content);
        }catch (Exception e){
            logger.error("打印日志出错",e);
        }
        return point.proceed();
    }
}
