package com.suntray.storage.lock;

import com.suntray.storage.config.SpringUtils;
import com.suntray.storage.domain.Dblock;
import com.suntray.storage.mapper.DblockMapper;
import com.suntray.utils.NetUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.Calendar;
import java.util.Map;
import java.util.Objects;
import java.util.concurrent.ConcurrentHashMap;
import java.util.concurrent.TimeUnit;
import java.util.concurrent.locks.Condition;
import java.util.concurrent.locks.Lock;

/**
 * 基于数据库的分布式锁
 */
public class DbLock implements Lock {
    private static final Logger log = LoggerFactory.getLogger(DbLock.class);
    private boolean interrupted;
    private String name;
    private String remark;
    private long expireTime;
    static Map<String, DbLock> locks = new ConcurrentHashMap<>();

    /**
     * 加锁
     *
     * @param name 锁名，唯一的
     */
    public DbLock(String name) {
        this(name, null);
    }

    /**
     * 加锁
     *
     * @param name        锁名，唯一的
     * @param holdSeconds 加锁时间，单位秒
     */
    public DbLock(String name, Integer holdSeconds) {
        this(name, holdSeconds, null);
    }

    /**
     * 加锁
     *
     * @param name        锁名，唯一的
     * @param holdSeconds 加锁时间，单位秒
     * @param remark      备注
     */
    public DbLock(String name, Integer holdSeconds, String remark) {
        this.name = name;
        this.remark = remark;
        if (holdSeconds == null) {
            holdSeconds = 60;
        }
        Calendar c = Calendar.getInstance();
        c.add(Calendar.SECOND, holdSeconds);
        this.expireTime = c.getTimeInMillis();
    }

    @Override
    public void lock() {
        while (!tryLock()) {
            if (interrupted) {
                throw new RuntimeException("interrupted");
            }
            try {
                TimeUnit.SECONDS.sleep(1);
            } catch (InterruptedException e) {
                log.error(e.getMessage());
                //e.printStackTrace()
            }
        }
    }

    @Override
    public void lockInterruptibly() throws InterruptedException {
        this.interrupted = true;
    }

    @Override
    public boolean tryLock() {
        try {
            DblockMapper dao = SpringUtils.getBean(DblockMapper.class);
            Dblock record = new Dblock();
            record.setRs(name);
            record.setRemark(remark);
            Calendar c = Calendar.getInstance();
            c.setTimeInMillis(expireTime);
            record.setExpireTime(c.getTime());
            record.setRowner(NetUtils.getMacAddr());
            dao.insert(record);
            locks.put(name, this);
            return true;
        } catch (Exception e) {
            return false;
        }
    }

    @Override
    public boolean tryLock(long time, TimeUnit unit) throws InterruptedException {
        long wait = TimeUnit.MILLISECONDS.convert(time, unit);
        while (!tryLock()) {
            if (interrupted) {
                throw new InterruptedException();
            }
            if (wait < System.currentTimeMillis()) {
                break;
            }
            TimeUnit.SECONDS.sleep(1);
        }
        return false;
    }

    @Override
    public void unlock() {
        DblockMapper dao = SpringUtils.getBean(DblockMapper.class);
        dao.deleteById(name);
        locks.remove(name);
    }

    @Override
    public Condition newCondition() {
        throw new UnsupportedOperationException();
    }

    /**
     * 查询锁是否存在
     *
     * @param name 锁名
     * @return
     */
    public static DbLock getByName(String name) {
        return locks.get(name);
    }

    public String getName() {
        return name;
    }

    public void setName(String name) {
        this.name = name;
    }

    public String getRemark() {
        return remark;
    }

    public void setRemark(String remark) {
        this.remark = remark;
    }

    public long getExpireTime() {
        return expireTime;
    }

    public void setExpireTime(long expireTime) {
        this.expireTime = expireTime;
    }

    public boolean isExpired() {
        return expireTime < System.currentTimeMillis();
    }

    @Override
    public boolean equals(Object o) {
        if (this == o) return true;
        if (o == null || getClass() != o.getClass()) return false;
        DbLock dbLock = (DbLock) o;
        return Objects.equals(name, dbLock.name);
    }

    @Override
    public int hashCode() {
        return Objects.hash(name);
    }
}
